import { Request, Response } from 'express'

import { BaseController } from '#application/base/baseController'
import MapObjectRepository from '#repositories/mapObjectRepository'
import MapRepository from '#repositories/mapRepository'
import SpriteRepository from '#repositories/spriteRepository'
import TileRepository from '#repositories/tileRepository'

export class CacheController extends BaseController {
  /**
   * Serve a list of maps and send as JSON
   * @param req
   * @param res
   */
  public async maps(req: Request, res: Response) {
    const items: any[] = []

    const mapRepository = new MapRepository()
    const maps = await mapRepository.getAll()

    for (const map of maps) {
      items.push(await map.cache())
    }

    return this.sendSuccess(res, items)
  }

  /**
   * Serve a list of map objects and send as JSON
   * @param req
   * @param res
   */
  public async mapObjects(req: Request, res: Response) {
    const items: any[] = []

    const mapObjectRepository = new MapObjectRepository()
    const mapObjects = await mapObjectRepository.getAll()

    for (const mapObject of mapObjects) {
      items.push(await mapObject.cache())
    }

    return this.sendSuccess(res, items)
  }
}
