import { Application } from 'express'

import { AssetsController } from '#controllers/assets'
import { AuthController } from '#controllers/auth'
import { AvatarController } from '#controllers/avatar'
import { CacheController } from '#controllers/cache'
import { TexturesController } from '#controllers/textures'

/**
 * HTTP manager
 */
class HttpManager {
  private readonly authController: AuthController = new AuthController()
  private readonly avatarController: AvatarController = new AvatarController()
  private readonly assetsController: AssetsController = new AssetsController()
  private readonly texturesController: TexturesController = new TexturesController()
  private readonly cacheController: CacheController = new CacheController()

  /**
   * Initialize HTTP manager
   * @param app
   */
  public async boot(app: Application) {
    // Add routes
    await this.addRoutes(app)
  }

  private async addRoutes(app: Application) {
    // Auth routes
    app.post('/login', (req, res) => this.authController.login(req, res))
    app.post('/register', (req, res) => this.authController.register(req, res))
    app.post('/reset-password-request', (req, res) => this.authController.requestPasswordReset(req, res))
    app.post('/reset-password', (req, res) => this.authController.resetPassword(req, res))

    // Avatar routes
    app.get('/avatar/:characterName', (req, res) => this.avatarController.getByName(req, res))
    app.get('/avatar/s/:characterTypeId/:characterHairId?', (req, res) => this.avatarController.getByParams(req, res))

    // Assets routes
    app.get('/assets/list_tiles', (req, res) => this.assetsController.listTiles(req, res))
    app.get('/assets/list_tiles/:mapId', (req, res) => this.assetsController.listTilesByMap(req, res))
    app.get('/assets/list_sprite_actions/:spriteId', (req, res) => this.assetsController.listSpriteActions(req, res))

    // Download texture file
    app.get('/textures/:type/:spriteId?/:file', (req, res) => this.texturesController.download(req, res))

    // Cache routes
    app.get('/cache/maps', (req, res) => this.cacheController.maps(req, res))
    app.get('/cache/map_objects', (req, res) => this.cacheController.mapObjects(req, res))
  }
}

export default new HttpManager()
